@@ -9,10 +9,12 @@ from rdfhelpers.rdfhelpers import expandQName, URI, getvalue, setvalue
 from rdfhelpers.rdfhelpers import isContainerItemPredicate, makeContainerItemPredicate, diff
 from rdfhelpers.cbd import cbd, reverse_cbd
 from rdfhelpers.rdfhelpers import getContainerStatements, getContainerItems, setContainerItems
+from rdfhelpers.rdfhelpers import getCollectionItems, makeCollection
 from rdfhelpers.rdfhelpers import SPARQLRepository, FocusedGraph, CBDGraph
 from rdfhelpers.templated import graphFrom, identity, mapDict
 from rdfhelpers.templated import Templated, Composable
 from rdfhelpers.constructor import Constructor
+from rdfhelpers.labels import GENERIC_PREFIX_MATCHER, abbreviate
 from rdfhelpers.labels import LabelCache, SKOSLabelCache, BNodeTracker, BNodeMarker
 from rdfhelpers.gsp import GraphStoreClient
 from rdfhelpers.tinyrml import RR, Mapper
@@ -22,10 +24,12 @@ __all__ = [
     'isContainerItemPredicate', 'makeContainerItemPredicate', 'diff',
     'cbd', "reverse_cbd",
     'getContainerStatements', 'getContainerItems', 'setContainerItems',
+    'getCollectionItems', 'makeCollection',
     'SPARQLRepository', 'FocusedGraph', 'CBDGraph',
     'graphFrom', 'identity', 'mapDict',
     'Templated', 'Composable',
     'Constructor',
+    'GENERIC_PREFIX_MATCHER', 'abbreviate',
     'LabelCache', 'SKOSLabelCache', 'BNodeTracker', 'BNodeMarker',
     'GraphStoreClient',
     'RR', 'Mapper'
