@@ -7,6 +7,7 @@
 #
 
 from rdflib import URIRef, BNode, Variable, Graph, RDF, RDFS, ConjunctiveGraph
+from rdflib.paths import ZeroOrMore
 from rdflib.plugins.stores.sparqlstore import SPARQLUpdateStore
 from rdfhelpers.templated import Templated, graphFrom, TemplatedQueryMixin
 from rdfhelpers.cbd import cbd
@@ -106,6 +107,25 @@ def setContainerItems(graph, node, predicate, values, newtype=RDF.Seq):
             graph.remove((node, predicate, container))
             graph.remove((container, None, None))
 
+def getCollectionItems(graph, collection):
+    return graph.objects(collection, (RDF.rest*ZeroOrMore)/RDF.first)
+
+def makeCollection(graph, items):
+    if items:
+        head = BNode()
+        prev = None
+        current = head
+        for item in items:
+            if prev:
+                graph.add((prev, RDF.rest, current))
+            graph.add((current, RDF.first, item))
+            prev = current
+            current = BNode()
+        graph.add((prev, RDF.rest, RDF.nil))
+        return head
+    else:
+        return None
+
 # FOCUSED GRAPH
 #
 # Instances of FocusedGraph have a specified "focus node".
@@ -114,7 +134,12 @@ class FocusedGraph(Graph):
     def __init__(self, focus=None, source=None, focus_class=None, **kwargs):
         super().__init__(**kwargs)
         if source:
-            self.parse(source)
+            if isinstance(source, str):
+                self.parse(source)
+            elif isinstance(source, list):
+                graphFrom(source, self)
+            else:
+                raise TypeError("Cannot use {} as a graph source".format(source))
         self._focus = focus or self.findFocus(focus_class=focus_class, **kwargs)
 
     @property
