@@ -3,6 +3,7 @@ import urllib.parse
 from rdflib import Literal, URIRef, BNode, Graph, RDF, Namespace
 
 RR = Namespace("http://www.w3.org/ns/r2rml#")
+RRE = Namespace("https://somanyaircraft.com/data/schemata/rdfhelpers/1/rre#")
 
 class URITemplateFormatter(string.Formatter):
     def get_value(self, key, args, kwargs):
@@ -29,8 +30,9 @@ class Node:
         return str(v) if v and interpret_literals and isinstance(v, Literal) else v
 
 class Template:
-    def __init__(self, template_string, ignore_field_keys=None):
+    def __init__(self, template_string, ignore_field_keys=None, empty_string_is_none=True):
         self.template_string = template_string
+        self.empty_string_is_none = empty_string_is_none
         self.fields = [f for _, f, _, _ in self.uri_formatter.parse(template_string)]
         self.candidate_field = None
         for f in self.fields:
@@ -43,16 +45,19 @@ class Template:
     uri_formatter = URITemplateFormatter()
     literal_formatter = string.Formatter()
 
-    def expand(self, data, term_type):
+    def expand(self, data, term_type, role):
         value = data[self.candidate_field]
-        if value is None:
-            return None
+        if value is None or (self.empty_string_is_none and value == ""):
+            # If there is no value (None or potentially an empty string), we return None, except if
+            # we are expanding for a subject map: in that case we must return a blank node, or there
+            # would be no subject at all...
+            return BNode() if role == RR.subjectMap else None
         elif isinstance(value, list):
             results = list()
             data2 = dict(data)
             for v in value:
                 data2[self.candidate_field] = v
-                results.append(self.expand(data2, term_type))
+                results.append(self.expand(data2, term_type, role))
             return results
         else:
             formatter = self.uri_formatter if term_type == RR.IRI else self.literal_formatter
@@ -64,6 +69,8 @@ class TermMap(Node):
         self.role = role
         self.column = None
         self.template = None
+        self.expression = None
+        self.globals = mapper.globals
         self.term_type = RR.Literal
         self.datatype = None
         self.language = None
@@ -78,9 +85,17 @@ class TermMap(Node):
                 template_string = self.value(RR.template)
                 if template_string:
                     self.template = Template(template_string,
-                                             ignore_field_keys=mapper.ignore_field_keys)
+                                             ignore_field_keys=mapper.ignore_field_keys,
+                                             empty_string_is_none=mapper.empty_string_is_none)
                 else:
-                    raise ValueError("Either rr:column or rr:template must be specified")
+                    expression = self.value(RRE.expression)
+                    if expression:
+                        if mapper.allow_expressions:
+                            self.expression = compile(expression, mapper.source_file, mode="eval")
+                        else:
+                            raise ValueError("No rre:expression allowed")
+                    else:
+                        raise ValueError("Specify rr:column, rr:template, or rre:expression")
         if self.node:
             self.classes = self.values(RR["class"])
             self.term_type = self.determineTermType()
@@ -102,7 +117,9 @@ class TermMap(Node):
         elif self.column:
             terms = self.termContent2term(data[self.column])
         elif self.template:
-            terms = self.termContent2term(self.template.expand(data, self.term_type))
+            terms = self.termContent2term(self.template.expand(data, self.term_type, self.role))
+        elif self.expression:
+            terms = self.termContent2term(eval(self.expression, self.globals, data))
         else:
             terms = [BNode()]
         if self.role == RR.subjectMap:
@@ -122,7 +139,7 @@ class TermMap(Node):
                 terms += self.termContent2term(v)
             return terms
         elif self.term_type == RR.IRI:
-            return [URIRef(content)]
+            return [content] if isinstance(content, BNode) else [URIRef(content)]
         elif self.term_type == RR.Literal:
             return [Literal(content, datatype=self.datatype, lang=self.language)]
         else:
@@ -167,11 +184,15 @@ class TriplesMap(Node):
         return result_graph
 
 class Mapper:
-    def __init__(self, mapping, triples_map_uri=None, ignore_field_keys=None):
+    def __init__(self, mapping,
+                 triples_map_uri=None, ignore_field_keys=None, empty_string_is_none=True,
+                 allow_expressions=True, global_bindings={}):
         if isinstance(mapping, Graph):
             graph = mapping
+            self.source_file = "<unknown>"
         else:
             graph = Graph()
+            self.source_file = mapping
             graph.parse(mapping)
         if triples_map_uri is None:
             for u in graph.subjects(RDF.type, RR.TriplesMap):
@@ -181,6 +202,11 @@ class Mapper:
                 raise ValueError("No rr:TriplesMap found")
         self.graph = graph
         self.ignore_field_keys = ignore_field_keys or {}
+        self.empty_string_is_none = empty_string_is_none
+        self.allow_expressions = allow_expressions
+        self.globals = (global_bindings
+                        if global_bindings.get("__builtins__", None)
+                        else {"__builtins__": __builtins__, **global_bindings})
         self.triples_map = TriplesMap(triples_map_uri, self)
 
     def query(self, database, filter_expr=None, result_graph=None):
